package de.intevation.flys.client.shared.model;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

import java.io.Serializable;

public class ToLoad implements Serializable
{
    public static final String SYNTHETIC_KEY = "key-";

    protected Map<String, Map<String, ArtifactFilter>> artifacts;

    public ToLoad() {
        artifacts = new HashMap<String, Map<String, ArtifactFilter>>();
    }

    public static final String uniqueKey(Map<?, ?> map) {
        int idx = map.size();

        String key = SYNTHETIC_KEY + idx;
        while (map.containsKey(key)) {
            key = SYNTHETIC_KEY + ++idx;
        }
        return key;
    }

    public void add(
        String artifactName,
        String factory,
        String out,
        String name,
        String ids
    ) {
        if (artifactName == null) {
            artifactName = uniqueKey(artifacts);
        }

        Map<String, ArtifactFilter> artifact = artifacts.get(artifactName);

        if (artifact == null) {
            artifact = new HashMap<String, ArtifactFilter>();
            artifacts.put(artifactName, artifact);
        }

        ArtifactFilter filter = artifact.get(factory);
        if (filter == null) {
            filter = new ArtifactFilter(factory);
            artifact.put(factory, filter);
        }

        filter.add(out, name, ids);
    }

    public boolean isEmpty() {
        return artifacts.isEmpty();
    }

    public List<Recommendation> toRecommendations() {
        List<Recommendation> recommendations = new ArrayList<Recommendation>();

        for (Map.Entry<String, Map<String, ArtifactFilter>> all:
            artifacts.entrySet()
        ) {
            String masterArtifact = all.getKey();

            if (masterArtifact.startsWith(SYNTHETIC_KEY)) { // system data
                masterArtifact = null;
            }

            for (Map.Entry<String, ArtifactFilter> entry:
                all.getValue().entrySet()
            ) {
                String         factory        = entry.getKey();
                ArtifactFilter artifactFilter = entry.getValue();

                String                ids;
                Recommendation.Filter filter;

                if (masterArtifact == null) { // system data
                    ids    = artifactFilter.collectIds();
                    filter = null;
                }
                else { // user specific
                    ids    = null;
                    filter = artifactFilter.toFilter();
                }

                Recommendation recommendation = new Recommendation(
                    factory, ids, masterArtifact, filter);

                recommendations.add(recommendation);
            }
        }

        return recommendations;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
